package cn.iocoder.yudao.module.system.service.robot.job;

import cn.iocoder.yudao.module.mqtt.api.task.dto.RobotAcceptTaskDTO;
import cn.iocoder.yudao.module.mqtt.api.task.dto.RobotAcceptTaskData;
import cn.iocoder.yudao.module.mqtt.enums.path.TaskTypeEnum;
import cn.iocoder.yudao.module.system.constant.robot.RobotTopicConstant;
import cn.iocoder.yudao.module.system.controller.admin.robot.task.TaskPPDistribution;
import cn.iocoder.yudao.module.system.dal.dataobject.houselocation.WareHouseLocationDO;
import cn.iocoder.yudao.module.system.dal.dataobject.robot.RobotTaskDetailDO;
import cn.iocoder.yudao.module.system.dal.mysql.robot.RobotTaskDetailMapper;
import cn.iocoder.yudao.module.system.enums.robot.RobotTaskTypeEnum;
import cn.iocoder.yudao.module.system.enums.robot.task.RobotTaksOrderTypeEnum;
import cn.iocoder.yudao.module.system.service.robot.RobotInformationService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.module.system.config.SystemJobConfiguration.NOTIFY_THREAD_POOL_TASK_EXECUTOR;

@Service
@Slf4j
public class RobotCommonTaskServiceImpl implements RobotCommonTaskService {

    @Resource
    private RobotTaskDetailMapper taskDetailMapper;

    @Autowired
    private RobotInformationService robotInformationService;

    /**
     * 让机器人自动充电
     *
     * @param robotNo
     */
    @Override
    @Async(NOTIFY_THREAD_POOL_TASK_EXECUTOR)
    public void startAutoCharge(String robotNo) {
        //todo 后续沟通下手动模式的需不需要自动充电


    }

    /**
     * 客户下发的任务
     *
     * @param list
     */
    @Override
    @Async(NOTIFY_THREAD_POOL_TASK_EXECUTOR)
    public void sendTaskToRobot(List<TaskPPDistribution> list) {
        List<Long> detailIds = list.stream().map(TaskPPDistribution::getId).collect(Collectors.toList());
        List<RobotTaskDetailDO> taskDetailDOS = taskDetailMapper.selectBatchIds(detailIds);
        Map<Long, RobotTaskDetailDO> locationDOMap =
                taskDetailDOS.stream().collect(Collectors.toMap(v -> v.getId(), Function.identity()));

        List<RobotAcceptTaskDTO> robotTaskDOS = new ArrayList<>();

        for (TaskPPDistribution v : list) {

            RobotTaskDetailDO robotTaskDetailDO = locationDOMap.get(v.getId());
            RobotAcceptTaskDTO robotTask = new RobotAcceptTaskDTO();
            robotTask.setOrder_id(v.getId().toString());

            String mac = robotInformationService.getMacByRobotNo(v.getRobotNo());
            robotTask.setTopic(RobotTopicConstant.ROBOT_TASK_TOPIC + mac);
            robotTask.setOrder_type(RobotTaksOrderTypeEnum.TASK.getType());

            if (RobotTaskTypeEnum.TAKE_RELEASE.getType().equals(robotTaskDetailDO.getTaskType())) {
                List<RobotAcceptTaskData> data = takeRelease(v,robotTaskDetailDO);
            }

        }


    }

    /**
     * 取放
     * @param v
     * @param robotTaskDetailDO
     * @return
     */
    private List<RobotAcceptTaskData> takeRelease(TaskPPDistribution v, RobotTaskDetailDO robotTaskDetailDO) {
        return null;
    }

    /**
     * 充电任务
     *
     * @param chargeList
     */
    @Override
    @Async(NOTIFY_THREAD_POOL_TASK_EXECUTOR)
    public void sendChargeTaskToRobot(List<TaskPPDistribution> chargeList) {

    }
}
